package UroIrmPackage;

import ij.*;
import java.util.*;

public class DicomHashtable extends Hashtable implements Header {

	private static int DATA_WIDTH =0, DATA_HEIGHT=1, DATA_TYPE=2,
			DATA_PERSONN=3,DATA_ACQDATE=4,DATA_ACQHOUR=5,DATA_PATIENTID=6;
	private static int nb_data = 3;

/** name of the 2 first label of the key */
	public static String SIZE ="size", TYPE ="type" ;
/** it is an subarray of STACK_TAG */
	public static final String[] KEY_TOKEN = {SIZE,TYPE,PERSONN,ACQDATE,ACQHOUR,STUDYDESCRIP};

// in case of an error while the creation of the table
	private static boolean error =false;
	private static String errorKey =null;

	public DicomHashtable(){
		super();
	}

/** tels whether the table was create with no error or not */
	public boolean hasError(){	return error;	}

/** give the erroneous keys separate by TOKEN_SEPARATOR */
	public String getErrorKey(){	return errorKey;	}

/** add an ImagePlus image in the current Hashtable */
	public boolean addImage(ImagePlus imp)throws NullKeyFieldException {
		if ( imp == null)
			return false;
		String key ="";
		NullKeyFieldException errorException =null;
		try {	key = createKey(imp); } 
		catch(NullKeyFieldException e){ error =true; errorException = e; key= errorException.errKey;}
		if ( key == null)
			return false;
		if ( !this.containsKey(key)){
			String[] data;
			this.put(key,new LinkedList());
			data = getData(imp);
			((LinkedList)(this.get(key))).add(stringArray2String(data));
		}
		((LinkedList)(this.get(key))).add(imp);
		if ( errorException!=null ) { throw (errorException);}
		return true;
	}

/** create the key (according to the data of the KEY_TOKEN ) for the table */	
	String createKey(ImagePlus imp) throws NullKeyFieldException {
		String key ="";
		HeaderDicom header = new HeaderDicom(imp);
		String tags = (String)imp.getProperty("Info");
		boolean[]exceptionIndice = new boolean[KEY_TOKEN.length];
		boolean erreur = false;
		int errCount =0;
		if ( tags == null)
			return null;
		for (int i = 0 ; i< KEY_TOKEN.length;i++){
			if ( (KEY_TOKEN[i]).equals(SIZE)) { 
					key+= i2s(imp.getWidth()) + " x " ;
					key += i2s(imp.getHeight())+ TOKEN_SEPARATOR;
			} else if ( (KEY_TOKEN[i]).equals(TYPE)) 
					key+= i2s(imp.getType())+ TOKEN_SEPARATOR;
			else {
				String aux = header.getTagImage(KEY_TOKEN[i]);
				if( aux!= null && !aux.equals("") ) {
					key += aux + TOKEN_SEPARATOR;
				} else {
					key += "null" + TOKEN_SEPARATOR;
					addErrorKey(KEY_TOKEN[i]);
					exceptionIndice[i] = true;
					erreur=true;
				}
			}
		}
		if (erreur )
			throw( new NullKeyFieldException(exceptionIndice,key));
		exceptionIndice=null;
		return key;
	}

/** add the new field  responsable of the error to the "errorKey" if this key 
	doesn't contain the field */  
	void addErrorKey(String value){
		String[] data = unmountKey(errorKey);
		if ( data == null) { errorKey = value + TOKEN_SEPARATOR; 
						return ;}
		for (int i=0;i<data.length;i++){	if( value.equals(data[i])) 
								return;}		
		errorKey += value+TOKEN_SEPARATOR; 
	}

/** gives an array presentation of the key. returns null if the key is null or 
 		does not exists */
	public String[] unmountKey(String Key){
		if ( Key ==null) return null;
		StringTokenizer chaine = new StringTokenizer(Key,TOKEN_SEPARATOR);
		return convertTokenizer2Array(chaine);		
	}
	
	private String[] getData(ImagePlus imp) {
			HeaderDicom header = new HeaderDicom(imp);
			nb_data = 3+ (STACK_TAG).length;
			String[] data = new String[nb_data];
			data[0]= i2s(imp.getWidth()) ; 
			data[1]= i2s(imp.getHeight()) ;
			data[2]= i2s(imp.getType()) ;
			int count = 3;
			for ( int i = 0; i<(STACK_TAG).length;i++){ 
				data[count] = header.getTagImage(STACK_TAG[i]);
				count ++;
			}
			return data;
	}
		
	
	private String stringArray2String(String[] data){
		String result="";
		for (int i =0;i<data.length;i++){
			result+=data[i]+TOKEN_SEPARATOR;
		}
		return result;
	}
	
	private String[] convertTokenizer2Array(StringTokenizer chaine){
		if ( chaine.countTokens() <= 0 )
			return null;
		String[] result = new String[chaine.countTokens()];
		for (int i =0; i< result.length;i++){
			result[i] = chaine.nextToken();
		}
		return result;
	}

	private String i2s(int i){
		return ((new Integer(i)).toString());
	}

/** returns the LinkedList (of ImagePlus) of the specific "key" */
	public LinkedList getList(String key){
		if ( !this.containsKey(key) )
			return null;
		return ((LinkedList)(this.get(key)));
	}


/** returns the informations of the LinkedList (of ImagePlus) destinated to the stack */
	public String[] getPreferencies(String key){
		LinkedList l = getList(key);
		String pref = (String)l.getFirst();
		StringTokenizer chaine = new StringTokenizer(pref,TOKEN_SEPARATOR);
		return convertTokenizer2Array(chaine);		
	}

/** returns the key of the specific "index" */
	public String getKey(int index){
		if ( index < 1 ||index > this.size())
			return null;
		Enumeration enum = this.keys();
		for ( int i =0;i<index-1;i++){
			enum.nextElement();
		}
		return (String)(enum.nextElement());		 
	}

/** return the specific information given by "Value" of the specific "Key" */
	public String getPrefValue(String key,int Value){
		String[] pref = (getPreferencies(key));
		if ( pref !=null)
			return pref[Value];
		return UNKNOWN;
	}

/** return the type of the stack */
	public String getType(String key){
		return ( getPrefValue(key,DATA_TYPE));
	}
	
/** return the width of the stack */
	public String getWidth(String key){
		return ( getPrefValue(key,DATA_WIDTH));
	}

/** return the height of the stack */
	public String getHeight(String key){
		return ( getPrefValue(key,DATA_HEIGHT));
	}
	
/** return the name of the patient of the stack */
	public String getName(String key){
		return ( getPrefValue(key,DATA_PERSONN));
	}
	
/** return the date of the examen of the stack */
	public String getDate(String key){
		return ( getPrefValue(key,DATA_ACQDATE));
	}
	
/** return the hour of the examen of the stack */
	public String getTime(String key){
		return ( getPrefValue(key,DATA_ACQHOUR));
	}
	
/** return the patient ID of the stack */
	public String getPatientID(String key){
		return ( getPrefValue(key,DATA_PATIENTID));
	}
}
